//
// This file was generated by the JavaTM Architecture for XML Binding(JAXB) Reference Implementation, v2.2.4
// See <a href="http://java.sun.com/xml/jaxb">http://java.sun.com/xml/jaxb</a>
// Any modifications to this file will be lost upon recompilation of the source schema.
// Generated on: 2016.09.28 at 06:51:34 PM CEST
//

// This needs to be in the src/main/java to ensure that the Namespace is mapped to the prefix "mods"
// this cannot be done by a gradle task at the moment
@javax.xml.bind.annotation.XmlSchema(namespace = "http://www.loc.gov/mods/v3", xmlns = {
        @XmlNs(prefix = "mods", namespaceURI = "http://www.loc.gov/mods/v3")}, elementFormDefault = javax.xml.bind.annotation.XmlNsForm.QUALIFIED)
package net.sf.jabref.logic.importer.fileformat.mods;

import javax.xml.bind.annotation.XmlNs;
