package net.sf.jabref.model.database.event;

import net.sf.jabref.model.metadata.event.MetaDataChangedEvent;

/**
 * This Event is automatically fired at the same time as {@link EntryEvent}, {@link GroupUpdatedEvent} or {@link MetaDataChangedEvent}.
 */
public class BibDatabaseContextChangedEvent {
    // no data
}
