package net.sf.jabref.gui;

/**
 * Defines the different modes that the BasePanel can operate in
 */

public enum BasePanelMode {
    SHOWING_NOTHING,
    SHOWING_PREVIEW,
    SHOWING_EDITOR,
    WILL_SHOW_EDITOR
}
