package net.sf.jabref.model.database.event;

/**
 * This Event is fired from {@link net.sf.jabref.logic.autosaveandbackup.AutosaveManager} in case that a save task is pending.
 */
public class AutosaveEvent {
    // no data
}
