package oracle.jdbc.dcn;

/**
 * A mocking class used as a placeholder for the real Oracle JDBC drivers to prevent build errors.
 */
public class DatabaseChangeRegistration {

    public void addListener(@SuppressWarnings("unused") DatabaseChangeListener listener) {
        // do nothing
    }
}
